"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _reactNative = require("react-native");

var _useMemoOne = require("use-memo-one");

var _react = _interopRequireWildcard(require("react"));

var _reactNativeGestureHandler = require("react-native-gesture-handler");

var _utils = require("../utils");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

const StackItem = _ref => {
  let {
    stack,
    zIndex,
    getParam,
    position,
    stackItem,
    openModal,
    closeModal,
    closeModals,
    hideBackdrop,
    currentModal,
    closeAllModals,
    eventListeners,
    clearListeners,
    registerListener,
    removeClosingAction,
    pendingClosingAction,
    wasOpenCallbackCalled,
    wasClosedByBackdropPress
  } = _ref;
  const {
    animatedValue,
    translateY
  } = (0, _useMemoOne.useMemo)(() => ({
    animatedValue: new _reactNative.Animated.Value(-1),
    translateY: new _reactNative.Animated.Value(0)
  }), []);
  const animatedListenerId = (0, _react.useRef)();
  const onCloseListener = (0, _react.useRef)(() => undefined);
  const {
    animationIn,
    animationOut,
    backBehavior,
    containerStyle,
    animateInConfig,
    animateOutConfig,
    transitionOptions,
    disableFlingGesture,
    position: verticalPosition
  } = (0, _useMemoOne.useMemo)(() => (0, _utils.getStackItemOptions)(stackItem, stack), [stack, stackItem]);
  (0, _react.useEffect)(() => {
    let onAnimateListener = () => undefined;

    if (transitionOptions && typeof transitionOptions !== 'function') {
      throw new Error(`'${stackItem.name}' transitionOptions should be a function. For instance:
      import ${stackItem.name} from './src/modals/${stackItem.name}';

      ...
      ${stackItem.name}: {
        modal: ${stackItem.name},
        transitionOptions: animatedValue => ({
          opacity: animatedValue.interpolate({
            inputRange: [0, 1, 2, 3],
            outputRange: [0, 1, 0.5, 0.25],
            extrapolate: 'clamp',
          }),
        }),
      },
      }`);
    }

    eventListeners.forEach(item => {
      if (item.event === `${stackItem.hash}_onAnimate`) {
        onAnimateListener = item.handler;
      } else if (item.event === `${stackItem.hash}_onClose`) {
        onCloseListener.current = item.handler;
      }
    });
    animatedListenerId.current = animatedValue.addListener(_ref2 => {
      let {
        value
      } = _ref2;
      return onAnimateListener(value);
    });
    return () => {
      animatedValue.removeAllListeners();
      clearListeners(stackItem.hash);
    };
  }, []);
  const updateAnimatedValue = (0, _useMemoOne.useCallback)((toValue, closeModalCallback, modalStackItemCallback) => {
    if (!closeModalCallback && animationIn) {
      animationIn(animatedValue, toValue, modalStackItemCallback);
    } else if (closeModalCallback && animationOut) {
      animationOut(animatedValue, toValue, () => {
        closeModalCallback(stackItem);
        modalStackItemCallback === null || modalStackItemCallback === void 0 ? void 0 : modalStackItemCallback();
      });
    } else {
      _reactNative.Animated.timing(animatedValue, {
        toValue,
        useNativeDriver: true,
        ...(closeModalCallback ? animateOutConfig : animateInConfig)
      }).start(_ref3 => {
        let {
          finished
        } = _ref3;

        if (finished) {
          closeModalCallback === null || closeModalCallback === void 0 ? void 0 : closeModalCallback(stackItem);
          modalStackItemCallback === null || modalStackItemCallback === void 0 ? void 0 : modalStackItemCallback();
        }
      });
    }
  }, [stackItem, animationIn, animationOut, animatedValue, animateInConfig, animateOutConfig]);
  const closeStackItem = (0, _useMemoOne.useCallback)((modalName, callback) => {
    const onCloseFns = () => {
      onCloseListener.current({
        type: 'closeModal',
        origin: wasClosedByBackdropPress ? 'backdrop' : 'default'
      });
      closeModal(modalName);
      callback === null || callback === void 0 ? void 0 : callback();
    };

    if (stack.openedItemsSize === 1) hideBackdrop();

    if (!modalName || modalName === currentModal) {
      updateAnimatedValue(position - 1, onCloseFns);
    } else onCloseFns();
  }, [currentModal, closeModal, hideBackdrop, onCloseListener, position, stack.openedItemsSize, updateAnimatedValue, wasClosedByBackdropPress]);
  const closeStackItems = (0, _useMemoOne.useCallback)((closingElement, callback) => {
    const onCloseFns = () => {
      onCloseListener.current({
        type: 'closeModals',
        origin: 'default'
      });
      let output = closeModals(closingElement);
      callback === null || callback === void 0 ? void 0 : callback();
      return output;
    };

    if (stack.openedItemsSize === 1) hideBackdrop();

    if (closingElement === currentModal && position === 1) {
      return updateAnimatedValue(position - 1, onCloseFns);
    } else return onCloseFns();
  }, [closeModals, currentModal, hideBackdrop, position, stack.openedItemsSize, updateAnimatedValue]);
  const closeAllStackItems = (0, _useMemoOne.useCallback)(callback => {
    if (stack.openedItemsSize === 1) hideBackdrop();
    updateAnimatedValue(position - 1, () => {
      onCloseListener.current({
        type: 'closeAllModals',
        origin: wasClosedByBackdropPress ? 'backdrop' : 'default'
      });
      closeAllModals();
      callback === null || callback === void 0 ? void 0 : callback();
    });
  }, [closeAllModals, hideBackdrop, position, stack.openedItemsSize, updateAnimatedValue, wasClosedByBackdropPress]);
  const onFling = (0, _useMemoOne.useCallback)(_ref4 => {
    let {
      nativeEvent
    } = _ref4;

    if (!disableFlingGesture && nativeEvent.oldState === _reactNativeGestureHandler.State.ACTIVE) {
      const toValue = verticalPosition === 'top' ? (0, _utils.vh)(-100) : (0, _utils.vh)(100);
      if (stack.openedItemsSize === 1) hideBackdrop();

      _reactNative.Animated.timing(translateY, {
        toValue,
        useNativeDriver: true,
        ...animateOutConfig
      }).start(_ref5 => {
        let {
          finished
        } = _ref5;

        if (finished) {
          onCloseListener.current({
            type: 'closeModal',
            origin: 'fling'
          });
          closeModal(stackItem);
        }
      });
    }
  }, [animateOutConfig, closeModal, disableFlingGesture, hideBackdrop, stack.openedItemsSize, stackItem, translateY, verticalPosition]);

  const renderAnimatedComponent = () => {
    const Component = stackItem.component;

    const addListener = (eventName, handler) => registerListener(stackItem.hash, eventName, handler);

    const removeAllListeners = () => clearListeners(stackItem.hash);

    return /*#__PURE__*/_react.default.createElement(_reactNative.Animated.View, {
      pointerEvents: "box-none",
      style: {
        transform: [{
          translateY
        }]
      }
    }, /*#__PURE__*/_react.default.createElement(_reactNativeGestureHandler.FlingGestureHandler, {
      direction: verticalPosition === 'top' ? _reactNativeGestureHandler.Directions.UP : verticalPosition === 'bottom' ? _reactNativeGestureHandler.Directions.DOWN : -1,
      onHandlerStateChange: onFling
    }, /*#__PURE__*/_react.default.createElement(_reactNative.Animated.View, {
      style: { ...(transitionOptions && transitionOptions(animatedValue))
      }
    }, /*#__PURE__*/_react.default.createElement(Component, {
      modal: {
        openModal,
        addListener,
        currentModal,
        removeAllListeners,
        closeModal: closeStackItem,
        closeModals: closeStackItems,
        closeAllModals: closeAllStackItems,
        getParam: (paramName, defaultValue) => getParam(stackItem.hash, paramName, defaultValue),
        ...(stackItem.params && {
          params: stackItem.params
        })
      }
    }))));
  };

  const justifyContent = (0, _useMemoOne.useMemo)(() => {
    switch (verticalPosition) {
      case 'top':
        return 'flex-start';

      case 'bottom':
        return 'flex-end';

      default:
        return 'center';
    }
  }, [verticalPosition]);
  (0, _react.useEffect)(() => {
    updateAnimatedValue(position, undefined, wasOpenCallbackCalled ? undefined : stackItem.callback);

    if (wasClosedByBackdropPress) {
      if (backBehavior === 'clear') closeAllStackItems();else closeStackItem(undefined);
    }
  }, [backBehavior, closeAllStackItems, closeStackItem, position, stackItem.callback, updateAnimatedValue, wasClosedByBackdropPress, wasOpenCallbackCalled]);
  (0, _react.useEffect)(() => {
    if (pendingClosingAction) {
      if (pendingClosingAction.action === 'closeModal') {
        closeStackItem(pendingClosingAction.modalName, pendingClosingAction.callback);
      } else if (pendingClosingAction.action === 'closeModals') {
        closeStackItems(pendingClosingAction.modalName, pendingClosingAction.callback);
      } else if (pendingClosingAction.action === 'closeAllModals') {
        closeAllStackItems(pendingClosingAction.callback);
      }

      removeClosingAction(pendingClosingAction);
    }
  }, [closeAllStackItems, closeStackItem, closeStackItems, pendingClosingAction, removeClosingAction]);
  return /*#__PURE__*/_react.default.createElement(_reactNative.Animated.View, {
    pointerEvents: "box-none",
    style: [styles.container, containerStyle, {
      justifyContent,
      zIndex
    }]
  }, renderAnimatedComponent());
};

const styles = _reactNative.StyleSheet.create({
  container: { ..._reactNative.StyleSheet.absoluteFillObject,
    alignItems: 'center',
    backgroundColor: 'transparent'
  }
});

var _default = /*#__PURE__*/(0, _react.memo)(StackItem, (prevProps, nextProps) => prevProps.position === nextProps.position && prevProps.stackItem.hash === nextProps.stackItem.hash && prevProps.pendingClosingAction === nextProps.pendingClosingAction && prevProps.wasOpenCallbackCalled === nextProps.wasOpenCallbackCalled && prevProps.wasClosedByBackdropPress === nextProps.wasClosedByBackdropPress);

exports.default = _default;
//# sourceMappingURL=StackItem.js.map