function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

import { useCallback, useMemo } from 'use-memo-one';
import React, { useEffect, useState, memo } from 'react';
import { Easing, Animated, StyleSheet, TouchableWithoutFeedback, Platform } from 'react-native';
import StackItem from './StackItem';
import { defaultOptions, getStackItemOptions, sh } from '../utils';

const ModalStack = props => {
  const {
    stack
  } = props;
  const [hasChangedBackdropColor, setBackdropColorStatus] = useState(false);
  const [backdropClosedItems, setBackdropClosedItems] = useState([]);
  const [openActionCallbacks, setOpenActionCallbacks] = useState([]);
  const {
    opacity,
    translateY
  } = useMemo(() => ({
    opacity: new Animated.Value(0),
    translateY: new Animated.Value(sh(100))
  }), []);
  const {
    backBehavior,
    backdropAnimationDuration,
    backdropColor,
    backdropOpacity
  } = useMemo(() => getStackItemOptions(Array.from(stack.openedItems).pop(), stack), [stack]);
  const hideBackdrop = useCallback(() => {
    Animated.timing(opacity, {
      toValue: 0,
      easing: Easing.inOut(Easing.ease),
      duration: backdropAnimationDuration,
      useNativeDriver: true
    }).start(_ref => {
      let {
        finished
      } = _ref;
      if (finished) translateY.setValue(sh(100));
    });
  }, [backdropAnimationDuration, opacity, translateY]);
  useEffect(() => {
    if (stack.openedItemsSize && backdropColor && backdropColor !== 'black' && !hasChangedBackdropColor) {
      setBackdropColorStatus(true);
    }
  }, [backdropColor, hasChangedBackdropColor, stack.openedItemsSize]);
  useEffect(() => {
    var _window$scrollY;

    const scrollY = Platform.OS === 'web' ? (_window$scrollY = window.scrollY) !== null && _window$scrollY !== void 0 ? _window$scrollY : document.documentElement.scrollTop : 0;

    if (stack.openedItemsSize) {
      translateY.setValue(scrollY);
      Animated.timing(opacity, {
        toValue: 1,
        easing: Easing.in(Easing.ease),
        duration: backdropAnimationDuration,
        useNativeDriver: true
      }).start();
    } else hideBackdrop();
  }, [backdropAnimationDuration, opacity, stack.openedItemsSize, translateY]);

  const renderStackItem = (stackItem, index) => {
    const position = stack.openedItemsSize - index;
    const pendingClosingAction = stack.pendingClosingActions.values().next().value;
    const hasPendingClosingAction = position === 1 && (pendingClosingAction === null || pendingClosingAction === void 0 ? void 0 : pendingClosingAction.currentModalHash) === stackItem.hash;
    return /*#__PURE__*/React.createElement(StackItem, _extends({}, props, {
      // @ts-ignore
      stackItem: stackItem,
      key: index,
      zIndex: index + 1,
      position: position,
      openModal: function () {
        // @ts-ignore
        props.openModal(...arguments);
        setOpenActionCallbacks(state => [...state, stackItem.hash]);
      },
      hideBackdrop: hideBackdrop,
      wasOpenCallbackCalled: openActionCallbacks.includes(stackItem.hash),
      wasClosedByBackdropPress: backdropClosedItems.includes(stackItem.hash),
      pendingClosingAction: hasPendingClosingAction ? pendingClosingAction : undefined
    }));
  };

  const renderStack = () => {
    if (!stack.openedItemsSize) return null;
    return [...stack.openedItems].map(renderStackItem);
  };

  const onBackdropPress = () => {
    if (backBehavior === 'none') return;
    const currentItem = [...stack.openedItems].slice(-1)[0];
    if (stack.openedItemsSize === 1) hideBackdrop();
    setBackdropClosedItems([...backdropClosedItems, currentItem === null || currentItem === void 0 ? void 0 : currentItem.hash]);
  };

  const renderBackdrop = () => {
    const onPress = () => onBackdropPress();

    const backgroundColor = stack.openedItemsSize && backdropColor ? backdropColor : hasChangedBackdropColor ? 'transparent' : 'black';
    return /*#__PURE__*/React.createElement(TouchableWithoutFeedback, {
      onPress: onPress
    }, /*#__PURE__*/React.createElement(Animated.View, {
      style: [styles.backdrop, {
        backgroundColor,
        opacity: opacity.interpolate({
          inputRange: [0, 1],
          outputRange: [0, backdropOpacity !== null && backdropOpacity !== void 0 ? backdropOpacity : defaultOptions.backdropOpacity]
        })
      }]
    }));
  };

  return /*#__PURE__*/React.createElement(Animated.View, {
    style: [styles.container, {
      opacity,
      transform: [{
        translateY
      }]
    }, Platform.OS === 'web' && stack.openedItemsSize ? styles.containerWeb : null]
  }, renderBackdrop(), renderStack());
};

const styles = StyleSheet.create({
  container: { ...Platform.select({
      android: { ...StyleSheet.absoluteFillObject,
        zIndex: 0
      },
      ios: { ...StyleSheet.absoluteFillObject,
        zIndex: 0
      }
    })
  },
  containerWeb: { ...StyleSheet.absoluteFillObject,
    overflow: 'hidden',
    height: '100vh',
    width: '100vw',
    zIndex: 0
  },
  backdrop: { ...StyleSheet.absoluteFillObject
  }
});
export default /*#__PURE__*/memo(ModalStack, (prevProps, nextProps) => prevProps.stack.openedItemsSize === nextProps.stack.openedItemsSize && prevProps.stack.pendingClosingActionsSize === nextProps.stack.pendingClosingActionsSize);
//# sourceMappingURL=ModalStack.js.map